/*
 *  ======== swi.c ========
 *  The swi example shows how to use the SYS/BIOS Swi module. It 
 *  demonstrates how the various Swi APIs work.
 */

#include <xdc/std.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Swi.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Semaphore.h>

#include <xdc/cfg/global.h>
#include "sys_dbg.h"
#include "reg_io.h"

typedef unsigned int u32;
typedef volatile void __iomem *addr_ptr;

/*
 * select the HS1 or HS2 for DCO Freq
 * return : CLKCTRL
 */
static u32 pll_dco_freq_sel(u32 clkout_dco)
{
	if (clkout_dco >= DCO_HS2_MIN && clkout_dco < DCO_HS2_MAX)
		return SELFREQDCO_HS2;
	else if (clkout_dco >= DCO_HS1_MIN && clkout_dco < DCO_HS1_MAX)
		return SELFREQDCO_HS1;
	else
		return 0;
}

/*
 * select the sigma delta config
 * return: sigma delta val
 */
static u32 pll_sigma_delta_val(u32 clkout_dco)
{
	u32 sig_val = 0;
	float frac_div;

	frac_div = (float) clkout_dco / 250;
	frac_div = frac_div + 0.90;
	sig_val = (int) frac_div;
	sig_val = sig_val << 24;

	return sig_val;
}

static void pll_config(u32 base, u32 n, u32 m, u32 m2, u32 clkctrl_val)
{
	u32 m2nval, mn2val, read_clkctrl = 0, clkout_dco = 0;
	u32 sig_val = 0, hs_mod = 0;

	m2nval = (m2 << 16) | n;
	mn2val = m;

	/* calculate clkout_dco */
	clkout_dco = ((OSC_0_FREQ / (n + 1)) * m);

	/* sigma delta & Hs mode selection skip for ADPLLS*/
	if (MODENA_PLL_BASE != base)
	{
		sig_val = pll_sigma_delta_val(clkout_dco);
		hs_mod = pll_dco_freq_sel(clkout_dco);
	}sys_dbg_info("before set ADPLLJ_CLKCTRL, pll_config, reg 0x%x, val 0x%08x, reg 0x%x, val 0x%08x\n",
			base + ADPLLJ_CLKCTRL, __raw_readl((addr_ptr)(base + ADPLLJ_CLKCTRL)),
			base + ADPLLJ_STATUS, __raw_readl((addr_ptr)(base + ADPLLJ_STATUS)));
	/* by-pass pll */
	read_clkctrl = __raw_readl((addr_ptr) (base + ADPLLJ_CLKCTRL));
	read_clkctrl = read_clkctrl | 0x00800000;
	sys_dbg_info("read_clkctrl= 0x%08x\n", read_clkctrl);
	__raw_writel(read_clkctrl, (addr_ptr) (base + ADPLLJ_CLKCTRL));
	sys_dbg_info("after set ADPLLJ_CLKCTRL, pll_config, reg 0x%x, val 0x%08x\n",
			base + ADPLLJ_CLKCTRL, __raw_readl((addr_ptr)(base + ADPLLJ_CLKCTRL)));
	while ((__raw_readl((addr_ptr) (base + ADPLLJ_STATUS)) & 0x101) != 0x101)
		;

	sys_dbg_info("after set ADPLLJ_CLKCTRL, pll_config, reg 0x%x, val 0x%08x\n",
			base + ADPLLJ_STATUS, __raw_readl((addr_ptr)(base + ADPLLJ_STATUS)));
	/* Clear TINITZ */
	read_clkctrl = __raw_readl((addr_ptr) (base + ADPLLJ_CLKCTRL));
	__raw_writel((read_clkctrl & 0xfffffffe),
			(addr_ptr) (base + ADPLLJ_CLKCTRL));

	/*
	 * ref_clk = 20/(n + 1);
	 * clkout_dco = ref_clk * m;
	 * clk_out = clkout_dco/m2;
	 */

	read_clkctrl = __raw_readl((addr_ptr) (base + ADPLLJ_CLKCTRL)) & 0xffffe3ff;
	__raw_writel(m2nval, (addr_ptr) (base + ADPLLJ_M2NDIV));
	__raw_writel(mn2val, (addr_ptr) (base + ADPLLJ_MN2DIV));

	/* Skip for modena(ADPLLS) */
	if (MODENA_PLL_BASE != base)
	{
		__raw_writel(sig_val, (addr_ptr) (base + ADPLLJ_FRACDIV));
		__raw_writel((read_clkctrl | hs_mod),
				(addr_ptr) (base + ADPLLJ_CLKCTRL));
	}

	/* Load M2, N2 dividers of ADPLL */
	__raw_writel(0x1, (addr_ptr) (base + ADPLLJ_TENABLEDIV));
	__raw_writel(0x0, (addr_ptr) (base + ADPLLJ_TENABLEDIV));

	/* Loda M, N dividers of ADPLL */
	__raw_writel(0x1, (addr_ptr) (base + ADPLLJ_TENABLE));
	__raw_writel(0x0, (addr_ptr) (base + ADPLLJ_TENABLE));

	/* configure CLKDCOLDOEN,CLKOUTLDOEN,CLKOUT Enable BITS */
	read_clkctrl = __raw_readl((addr_ptr) (base + ADPLLJ_CLKCTRL)) & 0xdfe5ffff;
	if (MODENA_PLL_BASE != base)
		__raw_writel((read_clkctrl | ADPLLJ_CLKCRTL_CLKDCO),
				(addr_ptr) (base + ADPLLJ_CLKCTRL));

	sys_dbg_info("before set ADPLLJ_CLKCTRL 0xff7fffff, pll_config, reg 0x%x, val 0x%08x, reg 0x%x, val 0x%08x\n",
			base + ADPLLJ_CLKCTRL, __raw_readl((addr_ptr)(base + ADPLLJ_CLKCTRL)),
			base+ADPLLJ_STATUS, __raw_readl((addr_ptr)(base + ADPLLJ_STATUS)));
	/* Enable TINTZ and disable IDLE(PLL in Active & Locked Mode */
	read_clkctrl = __raw_readl((addr_ptr) (base + ADPLLJ_CLKCTRL)) & 0xff7fffff;
	__raw_writel((read_clkctrl | 0x1), (addr_ptr) (base + ADPLLJ_CLKCTRL));
	sys_dbg_info("after set ADPLLJ_CLKCTRL 0xff7fffff, pll_config, reg 0x%x, val 0x%08x\n",
			base + ADPLLJ_CLKCTRL, __raw_readl((addr_ptr)(base + ADPLLJ_CLKCTRL)));
	/* Wait for phase and freq lock */
	while ((__raw_readl((addr_ptr) (base + ADPLLJ_STATUS)) & 0x600) != 0x600);

	sys_dbg_info("after set ADPLLJ_CLKCTRL 0xff7fffff, pll_config, reg 0x%x, val 0x%08x\n",
			base+ADPLLJ_STATUS, __raw_readl((addr_ptr)(base + ADPLLJ_STATUS)));
	sys_dbg_info("Finished pll_config\n");
}

void unlock_pll_control_mmr()
{
	/* ??? */
	__raw_writel(0x1EDA4C3D, (addr_ptr) 0x481C5040);
	__raw_writel(0x2FF1AC2B, (addr_ptr) 0x48140060);
	__raw_writel(0xF757FDC0, (addr_ptr) 0x48140064);
	__raw_writel(0xE2BC3A6D, (addr_ptr) 0x48140068);
	__raw_writel(0x1EBF131D, (addr_ptr) 0x4814006c);
	__raw_writel(0x6F361E05, (addr_ptr) 0x48140070);

}

void PLL_CLKOUT_ENABLE(u32 base)
{
	__raw_writel(__raw_readl(((addr_ptr) (base + CLKCTRL))) | 0x00100000,
			(addr_ptr) (base + CLKCTRL));
	while (((__raw_readl((addr_ptr) (base + STATUS))) & 0x00000020)
			!= 0x00000020)
		;
	__raw_writel(__raw_readl((addr_ptr) (base + CLKCTRL)) | 0x00000001,
			(addr_ptr) (base + CLKCTRL));
}

void PLL_Bypass(u32 base)
{
	__raw_writel(__raw_readl(((addr_ptr) (base + CLKCTRL))) | 0x00800000,
			(addr_ptr) (base + CLKCTRL));
	while (((__raw_readl((addr_ptr) (base + STATUS))) & 0x00000101)
			!= 0x00000101)
		;
	__raw_writel(__raw_readl((addr_ptr) (base + CLKCTRL)) & 0xfffffffe,
			(addr_ptr) (base + CLKCTRL));
}

void PLL_CLKOUT_DISABLE(u32 base)
{
	__raw_writel(__raw_readl(((addr_ptr) (base + CLKCTRL))) & 0xffefffff,
			(addr_ptr) (base + CLKCTRL));
	while (((__raw_readl((addr_ptr) (base + STATUS))) & 0x00000020)
			!= 0x00000000)
		;
	__raw_writel(__raw_readl((addr_ptr) (base + CLKCTRL)) & 0xfffffffe,
			(addr_ptr) (base + CLKCTRL));
}

static void dsp_pll_config()
{
	pll_config(DSP_PLL_BASE, DSP_N, DSP_M, DSP_M2, DSP_CLKCTRL);
}

void ipnc_ff_pll_init(int option)
{
	unlock_pll_control_mmr();
option = 1;
	if (option == 1)
	{
		PLL_CLKOUT_ENABLE(DSS_PLL_BASE);
		PLL_CLKOUT_ENABLE(VIDEO_0_PLL_BASE);
		PLL_CLKOUT_ENABLE(VIDEO_1_PLL_BASE);
		PLL_CLKOUT_ENABLE(HDMI_PLL_BASE);

		dsp_pll_config();
	}
	else
	{
		PLL_Bypass(VIDEO_0_PLL_BASE);
		PLL_CLKOUT_DISABLE(DSS_PLL_BASE);
		PLL_CLKOUT_DISABLE(VIDEO_0_PLL_BASE);
		PLL_CLKOUT_DISABLE(VIDEO_1_PLL_BASE);
		PLL_CLKOUT_DISABLE(HDMI_PLL_BASE);

		PLL_CLKOUT_DISABLE(DSP_PLL_BASE);
	}
}

/*
 *  ======== main ========
 */Void main()
{
	sys_dbg_info("Try to set ipnc_ff_pll_init\n");
	ipnc_ff_pll_init(1);

	sys_dbg_info("set pll finished\n");

	BIOS_start();
}
