/*****************************************************************************
**+------------------------------------------------------------------------+**
**|                                                                        |**
**|                Copyright 2010 Mistral Solutions Pvt Ltd.               |**
**|                                                                        |**
**|                                                                        |**
**|                                                                        |**   
**| This program is free software; you can redistribute it and/or          |**
**| modify it under the terms of the GNU General Public License as         |**
**| published by the Free Software Foundation; either version 2 of         |**
**| the License, or (at your option) any later version.                    |**
**|                                                                        |**
**| This program is distributed in the hope that it will be useful,        |**
**| but WITHOUT ANY WARRANTY; without even the implied warranty of         |**
**| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           |**
**| GNU General Public License for more details.                           |**
**|                                                                        |**      
**| You should have received a copy of the GNU General Public License      |**
**| along with this program; if not, write to the Free Software            |**
**| Foundation, Inc., 59 Temple Place, Suite 330, Boston,                  |**
**| MA 02111-1307 USA                                                      |**
**+------------------------------------------------------------------------+**
*****************************************************************************/ 

/**
 * \file    uart.c
 *
 * \brief   This file contains  Supporting routines for uart test application
 *
 * This file contains support  routines to test uart  
 * 
 *
 * \author  0043
 *
 * \version 01a,13aug,2010 Created
 */



#include "uart.h"
#include "stdio.h"
/** **************************************************************************
 * \n \brief Routine to get the value of a uart register.
 *
 * This routine get the value of a uart register.This routine takes uart 
 * controller base addresss as a parameter along with that register offset as
 * another parameter and returns the value of the register.
 * 
 *
 * \param   uart		[IN]	uart controller base address
 * \param   reg         [IN]    register offset value
 *  
 * \return
 * \n      returns register value
 * 
 */
UINT32 _UART_rget( UART_Handle uart, UINT32 reg )
{
    UINT32 reg_value;
    UINT32 lcr;
    UINT32 preg32     = ( ( UINT32 )uart ) + ( reg & ~( USE_LCR_80 | USE_LCR_BF ) );
    UINT32 set_lcr_80 = ( ( UINT32 )reg ) & USE_LCR_80;
    UINT32 set_lcr_bf = ( ( UINT32 )reg ) & USE_LCR_BF;

    /* Set LCR if needed */
    if ( set_lcr_80 )
    {
        lcr = _UART_rget( uart, LCR_OFFSET );
        _UART_rset( uart, LCR_OFFSET, 0x80 );
    }
    if ( set_lcr_bf )
    {
        lcr = _UART_rget( uart, LCR_OFFSET );
        _UART_rset( uart, LCR_OFFSET, 0xbf );
    }

    /* Get UART register */
    reg_value = *( volatile UINT32* )( preg32 );

    /* Return LCR reg if necessary */
    if ( set_lcr_80 || set_lcr_bf )
        _UART_rset( uart, LCR_OFFSET, lcr );

    return reg_value;
}
/** **************************************************************************
 * \n \brief Routine to set the value of a uart register.
 *
 * This routine set the value of a uart register.This routine takes uart 
 * controller base addresss as a parameter along with that register offset as
 * another parameter and the value of the register.
 * 
 *
 * \param   uart		[IN]	uart controller base address
 * \param   reg         [IN]    register offset value
 *  
 * 
 */
void _UART_rset( UART_Handle uart, UINT32 reg, UINT32 reg_value )
{
    UINT32 lcr;
    UINT32 preg32     = ( ( UINT32 )uart ) + ( reg & ~( USE_LCR_80 | USE_LCR_BF ) );
    UINT32 set_lcr_80 = ( ( UINT32 )reg ) & USE_LCR_80;
    UINT32 set_lcr_bf = ( ( UINT32 )reg ) & USE_LCR_BF;

    /* Set LCR if needed */
    if ( set_lcr_80 )
    {
        lcr = _UART_rget( uart, LCR_OFFSET );
        _UART_rset( uart, LCR_OFFSET, 0x80 );
    }
    if ( set_lcr_bf )
    {
        lcr = _UART_rget( uart, LCR_OFFSET );
        _UART_rset( uart, LCR_OFFSET, 0xbf );
    }

    /* Set UART register */
    *( volatile UINT32* )( preg32 ) = reg_value;

    /* Return LCR reg if necessary */
    if ( set_lcr_80 || set_lcr_bf )
        _UART_rset( uart, LCR_OFFSET, lcr );
}
/** **************************************************************************
 * \n \brief Routine to reset the uart  conntroller.
 *
 * This routine resets the uart controller.This routine takes uart 
 * controller base addresss as a parameter. 
 * 
 * \param   uart  [IN]   uart to be resetted
 *
 */
void _UART_reset( UART_Handle uart )
{
    /* Reset Sequence */
    _UART_rset( uart, EFR, 0x10 );
    _UART_rset( uart, IER_OFFSET, 0 );
    _UART_rset( uart, MCR_OFFSET, 0 );
    _UART_rset( uart, EFR, 0 );
    _UART_rset( uart, LCR_OFFSET, 0 );
    _UART_rset( uart, MDR1_OFFSET, 0x07 );
}
/** **************************************************************************
 * \n \brief Routine to open the uart .
 *
 * This routine enables the uart .This routine takes uart id  as a parameter 
 * which gives information about which uart to be selected  along with that
 * it takes baudrate as another parameter.
 * 
 *
 * \param   id		    [IN]    uart to be opened
 * \param   baudrate    [IN]    uart Baudrate 
 *
 *                                                                
 * \return        
 * \n        returns uart base address on success                                         
 * \n        returns 0 on failure
 *  
 * 
 */
UART_Handle _UART_open( UINT16 id, UINT32 baudrate )
{
    UART_Handle uart;
    UINT32 efr, ier, fcr, lcr, mcr, mdr1;

    switch ( id )
    {
        /* RS232 - UART */
        case UART_0:
            uart = DM814x_UART0_BASE;
            ier  = 0x00; // No interrupts enabled
            fcr  = 0x57; // FIFO Mode, 16 character trigger level
            lcr  = 0x03; // 8 bits, no parity, 1 stop
            mcr  = 0x00; //
            mdr1 = 0x00;
            break;

        /* Expansion */
        case UART_1:
            uart = DM814x_UART1_BASE;
            ier  = 0x00; // No interrupts enabled
            fcr  = 0x57; // FIFO Mode, 16 character trigger level
            lcr  = 0x03; // 8 bits, no parity, 1 stop
            mcr  = 0x00; //
            mdr1 = 0x00;
            break;

        /* IR Remote */
        case UART_2:
            /*uart = DM814x_UART2_BASE;
            ier  = 0x00; // No interrupts enabled
            fcr  = 0x00; // Non-FIFO Mode
            lcr  = 0x03; // 8 bits, no parity, 1 stop
            mcr  = 0x00; //
            mdr1 = 0x06; // CIR*/
            uart = DM814x_UART2_BASE;
            ier  = 0x00; // No interrupts enabled
            fcr  = 0x57; // FIFO Mode, 16 character trigger level
            lcr  = 0x03; // 8 bits, no parity, 1 stop
            mcr  = 0x00; //
            mdr1 = 0x00;
            break;

        default:
            return 0;
    }

    /* Reset the UART from any previous state */
    _UART_reset( uart );

    /* Set IER, FCR, LCR, MCR reg */
    efr = _UART_rget( uart, EFR );
    _UART_rset( uart, EFR, efr | 0x10 );
    _UART_rset( uart, IER_OFFSET, ier );
    _UART_rset( uart, FCR_OFFSET, fcr );
    _UART_rset( uart, LCR_OFFSET, lcr );
    _UART_rset( uart, MCR_OFFSET, mcr );
    _UART_rset( uart, EFR, efr );

    /* Set MDR1 - disable UART */
    _UART_rset( uart, MDR1_OFFSET, 0x07 );

    /* Set Baudrate */
    _UART_rset( uart, DLL, ( baudrate >> 0 ) & 0xff );
    _UART_rset( uart, DLH, ( baudrate >> 8 ) & 0xff );

    /* Set MDR1 - enable UART */
    _UART_rset( uart, MDR1_OFFSET, mdr1 );

    /* Return Handle */
    return uart;
}
/** **************************************************************************
 * \n \brief Routine  to close the uart .
 *
 * This routine disables the uart by resetting the uart.This routine takes uart 
 * id as a parameter which gives information about which uart to be selected  
 * 
 * 
 *
 * \param   uart		[IN]    uart to be opened
 * \param   baudrate    [IN]    uart Baudrate 
 *
 *                                                                
 * \return        
 * \n        returns uart base address on success                                         
 * \n        returns 0 on failure
 *  
 * 
 */
INT16 _UART_close( UART_Handle uart )
{
    _UART_reset( uart );
    return 0;
}
/** **************************************************************************
 * \n \brief Routine to check reception is ready in  uart 
 *
 * This routine checks whether RX FIFO is ready in uart.This routine takes uart 
 * id  as a parameter which gives information about which uart to be selected.  
 * 
 * 
 * \param   uart		    [IN]    uart base address
 * 
 *                                                                
 * \return
 * \n  returns  1 if  reception ready 
 * \n  returns  0 if reception is not ready
 */
 
INT16 _UART_rcvReady( UART_Handle uart )
{
    /* FIFO is empty */
    if ( _UART_rget( uart, LSR_OFFSET ) & 0x01 )
        return 1;
    
    /* FIFO is not empty */
           return 0;
   
}    

/** **************************************************************************
 * \n \brief Routine to get one byte of data
 *
 * This routine to get one byte of data.This routine takes uart id  as a 
 * parameter which gives information about which uart to be selected along with
 * that it contains a buffer pointer to copy the data.
 *
 * \param   uart		    [IN]    uart base address
 * \param   *data           [IN]    buffer pointer to copy the received data  
 *                                                                
 * \return
 * \n Returns 0 on successfull  completion of  the routine.
 */

INT16 _UART_getChar( UART_Handle uart, UINT8* data )
{
    /* Recv a byte */
    *data = _UART_rget( uart, RHR_OFFSET );
    return 0;
}
/** **************************************************************************
 * \n \brief Routine to check if transmission is ready in uart
 *
 * This routine to check if Tx Fifo is ready or not.This routine takes uart id  
 * as a parameter which gives information about which uart to be selected . 
 * 
 *
 * \param   uart		    [IN]    uart base address
 *  
 *                                                                
 * \return
 * \n Returns 0 if transmission is not ready
 * \n Returns 1 if transmission i ready
 */
INT16 _UART_xmtReady( UART_Handle uart )
{
    /* FIFO is empty */
    if ( _UART_rget( uart, LSR_OFFSET ) & 0x20 )
        return 1;

    /* FIFO is not empty */
        return 0;
   
}    
/** **************************************************************************
 * \n \brief Routine to send a byte of data to uart
 *
 * This routine sends one byte of data.This routine takes uart id  as a 
 * parameter which gives information about which uart to be selected along with
 * that it contains a buffer pointer to copy the data.
 *
 * \param   uart		    [IN]    uart base address.
 * \param   *data           [IN]    buffer pointer to be copied. 
 *                                                                
 * \return
 * \n Returns 0 on successfull  completion of  the routine.
 */

INT16 _UART_putChar( UART_Handle uart, UINT8 data )
{
    /* Send a byte */
    _UART_rset( uart, THR_OFFSET, data );
    return 0;
}


/** **************************************************************************
 * \n \brief Routine receive a byte of data from uart.
 *
 * This routine receive a byte of data from uart.This routine takes uart id  as
 * a parameter which gives information about which uart to be selected along with
 * that it takes bufferpointer as another parameter to copy the received data
 * and timeout as another parameter.
 *
 * \param   uart		 [IN]     uart to be selected
 * \param   *rx          [IN]     Buffer pointer to copy received data
 * \param   timeout      [IN]     Timeout for receiving
 *
 * \return
 * \n        returns 1 on failure
 * \n        returns 0 on success
 *
 *
 */
INT16 uart_recv(UART_Handle uart, UINT8* rx, INT32 timeout)
{
	/* Check if RX ready, if not timeout */
	while (_UART_rcvReady(uart) == 0)
	{

		if (timeout-- < 0)
		{
			printf("Timeout occured while Receiving\n");
			return 1;
		}
	}

	/* Read 1 byte */
	_UART_getChar(uart, rx);
	return 0;
}

/** **************************************************************************
 * \n \brief Routine send a byte of data to uart.
 *
 * This routine send a byte of data to uart.This routine takes uart id as
 * a parameter which gives information about which uart to be selected along with
 * that it takes buffer pointer as another parameter to send  data
 * and timeout as another parameter.
 *
 * \param   uart	    [IN]    uart to be selected
 * \param   tx          [IN]    Data to be transmitted
 * \param   timeout     [IN]    Timeout for transmitting
 *
 * \return
 * \n        returns 1 on failure
 * \n        returns 0 on success
 *
 *
 */
INT16 uart_send(UART_Handle uart, UINT8 tx, INT32 timeout)
{
	/* Check if TX ready, if not timeout */
	while (_UART_xmtReady(uart) == 0)
	{

		if (timeout-- < 0)
		{
			printf("Timeout occured while transmitting\n");
			return 1;
		}
	}

	/* Write 1 byte */
	_UART_putChar(uart, tx);

	return 0;
}
